@@ -39,10 +39,12 @@
 #include <QtCore/qmath.h>
 
 #define PHONON_CLASSNAME AudioOutput
+#define IFACES10 AudioOutputInterface410
+#define IFACES9 AudioOutputInterface49
 #define IFECES7 AudioOutputInterface47
 #define IFACES2 AudioOutputInterface42
 #define IFACES1 IFACES2
-#define IFACES0 AudioOutputInterface40, IFACES1, IFECES7
+#define IFACES0 AudioOutputInterface40, IFACES1, IFECES7, IFACES9, IFACES10
 #define PHONON_INTERFACENAME IFACES0
 
 namespace Phonon
@@ -227,27 +229,35 @@ bool AudioOutput::isMuted() const
 void AudioOutput::setMuted(bool mute)
 {
     P_D(AudioOutput);
-    if (d->muted != mute) {
-        PulseSupport *pulse = PulseSupport::getInstance();
-        if (mute) {
-            d->muted = mute;
-            if (k_ptr->backendObject()) {
-                if (pulse->isActive())
-                    pulse->setOutputMute(d->getStreamUuid(), mute);
-                else
-                    INTERFACE_CALL(setVolume(0.0));
-            }
-        } else {
-            if (k_ptr->backendObject()) {
-                if (pulse->isActive())
-                    pulse->setOutputMute(d->getStreamUuid(), mute);
-                else
-                    INTERFACE_CALL(setVolume(pow(d->volume, VOLTAGE_TO_LOUDNESS_EXPONENT)));
-            }
-            d->muted = mute;
-        }
-        emit mutedChanged(mute);
+
+    if (d->muted == mute) {
+        return;
     }
+    d->muted = mute;
+
+    if (!k_ptr->backendObject()) {
+        return;
+    }
+
+    PulseSupport *pulse = PulseSupport::getInstance();
+    if (pulse->isActive()) {
+        pulse->setOutputMute(d->getStreamUuid(), mute);
+    } else {
+        // When interface 9 is implemented we always default to it.
+        Iface<IFACES9> iface9(d);
+        if (iface9) {
+            iface9->setMuted(mute);
+            // iface9 is fully async, we let the backend emit the state change.
+            return;
+        }
+
+        if (mute) {
+            INTERFACE_CALL(setVolume(0.0));
+        } else {
+            INTERFACE_CALL(setVolume(pow(d->volume, VOLTAGE_TO_LOUDNESS_EXPONENT)));
+        }
+    }
+    emit mutedChanged(mute);
 }
 
 Category AudioOutput::category() const
@@ -301,6 +311,15 @@ void AudioOutputPrivate::setupBackendObject()
 
     QObject::connect(m_backendObject, SIGNAL(volumeChanged(qreal)), q, SLOT(_k_volumeChanged(qreal)));
     QObject::connect(m_backendObject, SIGNAL(audioDeviceFailed()), q, SLOT(_k_audioDeviceFailed()));
+    if (Iface<IFACES9>(this)) {
+        QObject::connect(m_backendObject, SIGNAL(mutedChanged(bool)),
+                         q, SLOT(_k_mutedChanged(bool)));
+    }
+
+    Iface<IFACES10> iface10(this);
+    if (iface10) {
+        iface10->setCategory(category);
+    }
 
     if (!PulseSupport::getInstance()->isActive()) {
         // set up attributes
