@@ -47,12 +47,15 @@
 
 #include "phononnamespace_p.h"
 #include "platform_p.h"
+#include "statesvalidator_p.h"
+
+#ifndef PHONON_NO_GRAPHICSVIEW
+#include <graphicsview/videographicsobject.h>
+#endif
 
 #define PHONON_CLASSNAME MediaObject
 #define PHONON_INTERFACENAME MediaObjectInterface
 
-QT_BEGIN_NAMESPACE
-
 namespace Phonon
 {
 PHONON_OBJECT_IMPL
@@ -110,6 +113,21 @@ void MediaObject::play()
 {
     P_D(MediaObject);
     if (d->backendObject() && isPlayable(d->mediaSource.type())) {
+
+    #ifndef PHONON_NO_GRAPHICSVIEW
+        VideoGraphicsObject *vgo = 0;
+        foreach (const Path &path, d->outputPaths) {
+            if (vgo = dynamic_cast<VideoGraphicsObject *>(path.sink())) {
+                // Play() is delayed until we had a paint event.
+                if (!vgo->canNegotiate()) {
+                    pWarning() << "VideoGraphicsObject not ready to be played because it has not received a paint() call yet, delaying playback.";
+                    connect(vgo, SIGNAL(gotPaint()), this, SLOT(play()));
+                    return;
+                }
+            }
+        }
+    #endif
+
         INTERFACE_CALL(play());
     }
 }
@@ -258,6 +276,8 @@ void MediaObject::setCurrentSource(const MediaSource &newSource)
     }
 #endif //QT_NO_PHONON_ABSTRACTMEDIASTREAM
 
+    d->playingQueuedSource = false;
+
     d->sendToZeitgeist(StoppedState);
     INTERFACE_CALL(setSource(d->mediaSource));
     d->sendToZeitgeist();
@@ -439,9 +459,15 @@ void MediaObjectPrivate::sendToZeitgeist(State eventState)
             break;
         }
 
+        QString eventManifestation;
+        if (playingQueuedSource)
+            eventManifestation = QZeitgeist::Manifestation::Event::ZGScheduledActivity;
+        else
+            eventManifestation = QZeitgeist::Manifestation::Event::ZGUserActivity;
+
         sendToZeitgeist(eventInterpretation,
-                        QZeitgeist::Manifestation::Event::ZGUserActivity,
-                        QLatin1Literal("app://" ) % Platform::applicationName() % QLatin1Literal(".desktop"),
+                        eventManifestation,
+                        QLatin1Literal("application://" ) % Platform::applicationName() % QLatin1Literal(".desktop"),
                         QDateTime::currentDateTime(),
                         mediaSource.url(),
                         title,
@@ -451,6 +477,7 @@ void MediaObjectPrivate::sendToZeitgeist(State eventState)
     }
     // Unset this so we don't send it again after a pause+play
     readyForZeitgeist = false;
+    playingQueuedSource = false;
 #else
     Q_UNUSED(eventState)
 #endif
@@ -589,7 +616,11 @@ void MediaObjectPrivate::_k_aboutToFinish()
 
     mediaSource = sourceQueue.head();
     readyForZeitgeist = false;
+    playingQueuedSource = true;
     pINTERFACE_CALL(setNextSource(mediaSource));
+
+    if (validator)
+        validator->sourceQueued();
 }
 
 void MediaObjectPrivate::_k_currentSourceChanged(const MediaSource &source)
@@ -617,12 +648,15 @@ void MediaObjectPrivate::setupBackendObject()
     qRegisterMetaType<MediaSource>("MediaSource");
     qRegisterMetaType<QMultiMap<QString, QString> >("QMultiMap<QString, QString>");
 
+    if (validateStates)
+        validator = new StatesValidator(q); // Parented, and non-invasive to MO.
+
 #ifndef QT_NO_PHONON_ABSTRACTMEDIASTREAM
-    QObject::connect(m_backendObject, SIGNAL(stateChanged(Phonon::State, Phonon::State)),
-                     q, SLOT(_k_stateChanged(Phonon::State, Phonon::State)), Qt::QueuedConnection);
+    QObject::connect(m_backendObject, SIGNAL(stateChanged(Phonon::State,Phonon::State)),
+                     q, SLOT(_k_stateChanged(Phonon::State,Phonon::State)), Qt::QueuedConnection);
 #else
-    QObject::connect(m_backendObject, SIGNAL(stateChanged(Phonon::State, Phonon::State)),
-                     q, SIGNAL(stateChanged(Phonon::State, Phonon::State)), Qt::QueuedConnection);
+    QObject::connect(m_backendObject, SIGNAL(stateChanged(Phonon::State,Phonon::State)),
+                     q, SIGNAL(stateChanged(Phonon::State,Phonon::State)), Qt::QueuedConnection);
 #endif // QT_NO_PHONON_ABSTRACTMEDIASTREAM
 #ifndef QT_NO_PHONON_VIDEO
     QObject::connect(m_backendObject, SIGNAL(hasVideoChanged(bool)),
@@ -643,8 +677,8 @@ void MediaObjectPrivate::setupBackendObject()
                      q, SIGNAL(prefinishMarkReached(qint32)), Qt::QueuedConnection);
     QObject::connect(m_backendObject, SIGNAL(totalTimeChanged(qint64)),
                      q, SIGNAL(totalTimeChanged(qint64)), Qt::QueuedConnection);
-    QObject::connect(m_backendObject, SIGNAL(metaDataChanged(QMultiMap<QString, QString>)),
-                     q, SLOT(_k_metaDataChanged(QMultiMap<QString, QString>)), Qt::QueuedConnection);
+    QObject::connect(m_backendObject, SIGNAL(metaDataChanged(QMultiMap<QString,QString>)),
+                     q, SLOT(_k_metaDataChanged(QMultiMap<QString,QString>)), Qt::QueuedConnection);
     QObject::connect(m_backendObject, SIGNAL(currentSourceChanged(MediaSource)),
                      q, SLOT(_k_currentSourceChanged(MediaSource)), Qt::QueuedConnection);
 
@@ -745,8 +779,6 @@ MediaObject *createPlayer(Phonon::Category category, const MediaSource &source)
 
 } //namespace Phonon
 
-QT_END_NAMESPACE
-
 #include "moc_mediaobject.cpp"
 
 #undef PHONON_CLASSNAME
